@@ -94,6 +94,9 @@ class XSessionManager:
                 sd.process_create_time = datetime.datetime.fromtimestamp(process.create_time()).strftime("%Y-%m-%d %H:%M:%S")
                 sd.cpu_percent = process.cpu_percent()
                 sd.memory_percent = process.memory_percent()
+                sd.window_state = sd.WindowState()
+                sd.window_state.is_above = wnck_utils.is_above(sd.window_id_the_int_type)
+                sd.window_state.is_sticky = wnck_utils.is_sticky(sd.window_id_the_int_type)
             except psutil.NoSuchProcess as e:
                 print('Failed to get process [%s] info using psutil due to: %s' % (sd, str(e)))
                 sd.app_name = ''
@@ -365,9 +368,13 @@ class XSessionManager:
         self._moving_windows_pool.apply_async(
             retry.Retry(6, 1).do_retry(self._move_window, (namespace_obj, pid)))
 
-    def _move_window(self, namespace_obj: XSessionConfigObject, pid: int = None, need_retry=True):
+    def _move_window(self, saved_window: XSessionConfigObject, pid: int = None, need_retry=True):
         try:
-            desktop_number = namespace_obj.desktop_number
+            desktop_number = saved_window.desktop_number
+            if hasattr(saved_window, 'window_state'):
+                saved_window_state = saved_window.window_state
+            else:
+                saved_window_state = None
 
             pids = []
             if pid:
@@ -376,7 +383,7 @@ class XSessionManager:
 
             # Get process info according to command line
             if len(pids) == 0:
-                cmd = namespace_obj.cmd
+                cmd = saved_window.cmd
                 if len(cmd) <= 0:
                     return
 
@@ -386,10 +393,10 @@ class XSessionManager:
 
                     if self._is_same_cmd(p, cmd):
                         pids.append(p.pid)
-                        break
+                        # break
 
             if len(pids) == 0:
-                self._windows_can_not_be_moved.append(namespace_obj)
+                self._windows_can_not_be_moved.append(saved_window)
                 return
 
             no_need_to_move = True
@@ -411,9 +418,11 @@ class XSessionManager:
             for running_window in x_session_config_objects:
                 if running_window.pid in pids:
                     no_need_to_compare_title = (counter[running_window.pid] == 1)
-                    if no_need_to_compare_title or self._is_same_window(running_window,
-                                                                        namespace_obj):
+                    if no_need_to_compare_title or\
+                            self._is_same_window(running_window,
+                                                 saved_window):
                         if running_window.desktop_number == int(desktop_number):
+                            self.fix_window_state(saved_window_state, running_window.window_id_the_int_type)
                             if not self._suppress_log_if_already_in_workspace:
                                 print('"%s" has already been in Workspace %s' % (running_window.window_title,
                                                                                  desktop_number))
@@ -428,21 +437,29 @@ class XSessionManager:
                     no_need_to_move = False
 
             if need_retry and len(moving_windows) == 0:
-                raise retry.NeedRetryException(namespace_obj)
+                raise retry.NeedRetryException(saved_window)
             elif no_need_to_move:
                 return
 
             for running_window in moving_windows:
                 running_window_id = running_window.window_id
+                window_id_the_int_type = running_window.window_id_the_int_type
                 if running_window_id in self._moved_windowids_cache:
+                    self.fix_window_state(saved_window_state, window_id_the_int_type)
                     continue
                 window_title = running_window.window_title
                 if string_utils.empty_string(window_title):
-                    window_title = wnck_utils.get_app_name(running_window.window_id_the_int_type)
+                    window_title = wnck_utils.get_app_name(window_id_the_int_type)
                 print('Moving window to desktop:           [%s : %s]' % (window_title, desktop_number))
                 # wmctl_wrapper.move_window_to(running_window_id, str(desktop_number))
-                wnck_utils.move_window_to(running_window.window_id_the_int_type, desktop_number)
+                is_sticky = wnck_utils.is_sticky(window_id_the_int_type)
+                wnck_utils.move_window_to(window_id_the_int_type, desktop_number)
+                # Wait some time for processing event completely, no guarantee though
+                sleep(0.25)
                 self._moved_windowids_cache.append(running_window_id)
+                self.fix_window_state(saved_window_state, window_id_the_int_type)
+                if not wnck_utils.is_sticky(window_id_the_int_type) and is_sticky:
+                    wnck_utils.stick(window_id_the_int_type)
                 # Wait some time to prevent 'X Error of failed request:  BadWindow (invalid Window parameter)'
                 # sleep(0.25)
         except retry.NeedRetryException as ne:
@@ -451,6 +468,15 @@ class XSessionManager:
             import traceback
             print(traceback.format_exc())
 
+    def fix_window_state(self,
+                         window_state: XSessionConfigObject.WindowState,
+                         window_id_the_int_type: int):
+        if window_state:
+            if window_state.is_sticky:
+                wnck_utils.stick(window_id_the_int_type)
+            if window_state.is_above:
+                wnck_utils.make_above(window_id_the_int_type)
+
     def _is_same_window(self, window1: XSessionConfigObject, window2: XSessionConfigObject):
         # Deal with JetBrains products. Move the window if they are the same project.
         app_name1 = wnck_utils.get_app_name(window1.window_id_the_int_type)
@@ -466,6 +492,9 @@ class XSessionManager:
     def _is_same_cmd(self, p: psutil.Process, second_cmd: List):
         first_cmdline = [c for c in p.cmdline() if (c != "--gapplication-service" and not c.startswith('--pid='))]
         second_cmd = [c for c in second_cmd if (c != "--gapplication-service" and not c.startswith('--pid='))]
+        if len(first_cmdline) <= 0 or len(second_cmd) <= 0:
+            return
+
         first_one_is_snap_app, first_snap_app_name = snapd_workaround.Snapd.is_snap_app(first_cmdline[0])
         if first_one_is_snap_app:
             second_one_also_is_snap_app, second_snap_app_name = snapd_workaround.Snapd.is_snap_app(second_cmd[0])
